"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getJSONSchemaModel = getJSONSchemaModel;
var _properties = require("./properties");
function getRelationFields(model) {
    return model.fields.map((field)=>{
        var ref;
        return field.relationName && !((ref = field.documentation) === null || ref === void 0 ? void 0 : ref.includes("@MongoSchema.omit")) ? field.name : "";
    }).filter(Boolean);
}
function getRequiredFields(model) {
    return model.fields.map((field)=>{
        var ref;
        return !field.relationName && field.isRequired && !((ref = field.documentation) === null || ref === void 0 ? void 0 : ref.includes("@MongoSchema.omit")) ? field.name : "";
    }).filter(Boolean).filter((field)=>field !== "id");
}
function getJSONSchemaModel(modelMetaData) {
    return (model)=>{
        const definitionPropsMap = model.fields.map((0, _properties).getJSONSchemaProperty(modelMetaData)).filter((item)=>Boolean(item));
        const propertiesMap = definitionPropsMap.map(([name, definition])=>[
                name,
                definition
            ]);
        const relationFields = getRelationFields(model);
        const requiredFields = getRequiredFields(model);
        const propertiesWithoutRelations = propertiesMap.filter((prop)=>relationFields.findIndex((field)=>field === prop[0]) === -1).filter((prop)=>prop[0] !== "id");
        const properties = Object.fromEntries(propertiesWithoutRelations);
        const definition = {
            bsonType: "object",
            properties,
            ...requiredFields.length ? {
                required: requiredFields
            } : {}
        };
        return [
            model.name,
            definition
        ];
    };
}
