"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getJSONSchemaProperty = getJSONSchemaProperty;
var _helpers = require("./helpers");
var _assertions = require("./assertions");
function getJSONSchemaScalar(fieldType) {
    switch(fieldType){
        case "Int":
            return "int";
        case "BigInt":
            return "long";
        case "DateTime":
            return "date";
        case "Bytes":
        case "String":
            return "string";
        case "Float":
        case "Decimal":
            return "int";
        case "Json":
            return [
                "int",
                "string",
                "bool",
                "object",
                "array",
                "null"
            ];
        case "Boolean":
            return "bool";
        default:
            (0, _helpers).assertNever(fieldType);
    }
}
function getJSONSchemaType(field) {
    var ref;
    const { isList  } = field;
    const isObjectId = (ref = field.documentation) === null || ref === void 0 ? void 0 : ref.includes("@db.ObjectId");
    const scalarFieldType = (0, _helpers).isScalarType(field) && !isList ? isObjectId ? "objectId" : getJSONSchemaScalar(field.type) : field.isList ? "array" : (0, _helpers).isEnumType(field) ? isObjectId ? "objectId" : "string" : "object";
    return scalarFieldType;
}
function getJSONSchemaForPropertyReference(field) {
    const notNullable = field.isRequired || field.isList;
    (0, _assertions).assertFieldTypeIsString(field.type);
    const ref = {
        $ref: field.type
    };
    return notNullable ? ref : {
        anyOf: [
            ref,
            {
                type: "null"
            }
        ]
    };
}
function getItemsByDMMFType(field) {
    var ref;
    const isObjectId = (ref = field.documentation) === null || ref === void 0 ? void 0 : ref.includes("@db.ObjectId");
    return (0, _helpers).isScalarType(field) && !field.isList || (0, _helpers).isEnumType(field) ? undefined : (0, _helpers).isScalarType(field) && field.isList ? {
        bsonType: isObjectId ? "objectId" : getJSONSchemaScalar(field.type)
    } : getJSONSchemaForPropertyReference(field);
}
function isSingleReference(field) {
    return !(0, _helpers).isScalarType(field) && !field.isList && !(0, _helpers).isEnumType(field);
}
function getEnumListByDMMFType(modelMetaData) {
    return (field)=>{
        const enumItem = modelMetaData.enums.find(({ name  })=>name === field.type);
        if (!enumItem) return undefined;
        return enumItem.values.map((item)=>item.name);
    };
}
function getDescription(field) {
    return field.documentation;
}
function getPropertyDefinition(modelMetaData, field) {
    const bsonType = getJSONSchemaType(field);
    const items = getItemsByDMMFType(field);
    const enumList = getEnumListByDMMFType(modelMetaData)(field);
    const description = getDescription(field);
    const definition = {
        bsonType,
        ...(0, _helpers).isDefined(items) && {
            items
        },
        ...(0, _helpers).isDefined(enumList) && {
            enum: enumList
        },
        ...(0, _helpers).isDefined(description) && {
            description
        }
    };
    return definition;
}
function getJSONSchemaProperty(modelMetaData) {
    return (field)=>{
        var ref, ref1, ref2;
        if ((ref = field.documentation) === null || ref === void 0 ? void 0 : ref.includes("@MongoSchema.omit")) {
            return null;
        }
        const propertyMetaData = {
            required: field.isRequired
        };
        const property = isSingleReference(field) ? getJSONSchemaForPropertyReference(field) : getPropertyDefinition(modelMetaData, field);
        const fieldName = ((ref1 = field.documentation) === null || ref1 === void 0 ? void 0 : ref1.includes("@map(")) ? ((ref2 = field.documentation.match(/@map\("(.+)"\)/)) === null || ref2 === void 0 ? void 0 : ref2[1]) || field.name : field.name;
        return [
            fieldName,
            property,
            propertyMetaData
        ];
    };
}
