"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.applyMongoSchema = void 0;
var _mongodb = require("mongodb");
var _getConfig = require("./getConfig");
const applyMongoSchema = async (schema)=>{
    const { mongoUri  } = (0, _getConfig).getConfig();
    const client = new _mongodb.MongoClient(mongoUri);
    try {
        await client.connect();
        const database = client.db();
        await Promise.all(Object.keys(schema.properties).map(async (collectionName)=>{
            try {
                await database.command({
                    collMod: collectionName,
                    validator: {
                        $jsonSchema: schema.properties[collectionName]
                    },
                    validationLevel: process.env.VALIDATION_LEVEL || "strict",
                    validationAction: process.env.VALIDATION_ACTION || "error"
                });
                console.info(`✅ Applied $jsonSchema to collection ${collectionName}`);
            } catch (e) {
                console.info(`❌ Failed to apply $jsonSchema to collection ${collectionName}`);
            }
        }));
    } finally{
        await client.close();
    }
};
exports.applyMongoSchema = applyMongoSchema;
