"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.run = run;
exports.getJsonSchema = void 0;
var _childProcess = require("child_process");
var _util = require("util");
var _fs = require("fs");
var _applyMongoSchema = require("./applyMongoSchema");
var _getConfig = require("./getConfig");
var _processSchema = require("./processSchema");
var _verifyMongoSchema = require("./verifyMongoSchema");
const exec = (0, _util).promisify(_childProcess.exec);
const getJsonSchema = async ()=>{
    const { jsonSchemaPath  } = (0, _getConfig).getConfig();
    const data = String(await _fs.promises.readFile(jsonSchemaPath));
    return JSON.parse(data);
};
exports.getJsonSchema = getJsonSchema;
async function run(command) {
    const { processedPrismaSchemaPath , jsonSchemaPath  } = (0, _getConfig).getConfig();
    // 1. pre-process the schema
    await (0, _processSchema).processSchema();
    // 2. execute yarn generate
    const { stderr  } = await exec(`yarn prisma generate --schema ${processedPrismaSchemaPath}`);
    if (stderr) {
        console.error("Prisma schema generation failed!", stderr);
    } else {
        console.info("Prisma schema generated");
    }
    // 3. run the actuall command
    const schema = await getJsonSchema();
    if (command === "apply") {
        await (0, _applyMongoSchema).applyMongoSchema(schema);
    }
    if (command === "verify") {
        await (0, _verifyMongoSchema).verifyMongoSchema(schema);
    }
    // 4. remove generated artifacts
    await _fs.promises.rm(processedPrismaSchemaPath);
    await _fs.promises.rm(jsonSchemaPath);
}
